@@ -11,7 +11,7 @@ import popper.utils as pu
 from popper.config import ConfigLoader
 from popper.parser import WorkflowParser
 from popper.runner import WorkflowRunner
-from popper.runner_host import HostRunner, DockerRunner, SingularityRunner
+from popper.runner_host import HostRunner, DockerRunner, PodmanRunner, SingularityRunner
 from popper.cli import log as log
 from .test_common import PopperTest
 
@@ -71,14 +71,14 @@ class TestHostHostRunner(PopperTest):
         pid, ecode, output = HostRunner._exec_cmd(cmd, logging=False)
         self.assertGreater(pid, 0)
         self.assertEqual(ecode, 0)
-        self.assertEqual(output, "hello-world\n")
+        self.assertEqual(output, "hello-world")
 
         with LogCapture("popper") as logc:
             pid, ecode, output = HostRunner._exec_cmd(cmd)
             self.assertGreater(pid, 0)
             self.assertEqual(ecode, 0)
             self.assertEqual(output, "")
-            logc.check_present(("popper", "STEP_INFO", "hello-world\n"))
+            logc.check_present(("popper", "STEP_INFO", "hello-world"))
 
         cmd = ["env"]
         pid, ecode, output = HostRunner._exec_cmd(
@@ -102,11 +102,11 @@ class TestHostHostRunner(PopperTest):
         self.assertRaises(ProcessLookupError, os.kill, pid, 0)
 
 
+@unittest.skipIf(os.environ.get("ENGINE", "docker") != "docker", "ENGINE != docker")
 class TestHostDockerRunner(PopperTest):
     def setUp(self):
         log.setLevel("CRITICAL")
 
-    @unittest.skipIf(os.environ.get("ENGINE", "docker") != "docker", "ENGINE != docker")
     def test_create_container(self):
         config = ConfigLoader.load()
         step = Box(
@@ -122,6 +122,7 @@ class TestHostDockerRunner(PopperTest):
             c = dr._create_container(cid, step)
             self.assertEqual(c.status, "created")
             c.remove()
+
         step = Box(
             {
                 "uses": "docker://alpine:3.9",
@@ -136,7 +137,21 @@ class TestHostDockerRunner(PopperTest):
             self.assertEqual(c.status, "created")
             c.remove()
 
-    @unittest.skipIf(os.environ.get("ENGINE", "docker") != "docker", "ENGINE != docker")
+        step = Box(
+            {
+                "uses": "docker://alpine:3.9",
+                "runs": ["echo", "hello_world"],
+                "options": {"volumes": ("/tmp:/bindmounteddir",)},
+                "id": "container",
+            },
+            default_box=True,
+        )
+        cid = pu.sanitized_name(step.id, config.wid)
+        with DockerRunner(init_docker_client=True, config=config) as dr:
+            c = dr._create_container(cid, step)
+            self.assertEqual(c.status, "created")
+            c.remove()
+
     def test_stop_running_tasks(self):
         with DockerRunner() as dr:
             dclient = docker.from_env()
@@ -151,127 +166,157 @@ class TestHostDockerRunner(PopperTest):
             self.assertEqual(c2.status, "created")
             dclient.close()
 
-    @unittest.skipIf(os.environ.get("ENGINE", "docker") != "docker", "ENGINE != docker")
-    def test_get_container_kwargs(self):
-        step = Box(
-            {
-                "uses": "popperized/bin/sh@master",
-                "args": ["ls"],
-                "id": "one",
-                "dir": "/tmp/",
-            },
-            default_box=True,
-        )
-
-        config_dict = {
-            "engine": {
-                "name": "docker",
-                "options": {
-                    "privileged": True,
-                    "hostname": "popper.local",
-                    "domainname": "www.example.org",
-                    "volumes": ["/path/in/host:/path/in/container"],
-                    "environment": {"FOO": "bar"},
-                },
-            },
-        }
-
-        config = ConfigLoader.load(
-            config_file=config_dict, workspace_dir="/path/to/workdir"
-        )
-
-        with DockerRunner(init_docker_client=False, config=config) as dr:
-            args = dr._get_container_kwargs(step, "alpine:3.9", "container_a")
-
-            self.assertEqual(
-                args,
-                {
-                    "image": "alpine:3.9",
-                    "command": ["ls"],
-                    "name": "container_a",
-                    "volumes": [
-                        "/path/to/workdir:/workspace",
-                        "/var/run/docker.sock:/var/run/docker.sock",
-                        "/path/in/host:/path/in/container",
-                    ],
-                    "working_dir": "/tmp/",
-                    "environment": {"FOO": "bar"},
-                    "entrypoint": None,
-                    "detach": True,
-                    "stdin_open": False,
-                    "tty": False,
-                    "privileged": True,
-                    "hostname": "popper.local",
-                    "domainname": "www.example.org",
-                },
-            )
-
-        # check container kwargs when pty is enabled
-        config = ConfigLoader.load(
-            config_file=config_dict, workspace_dir="/path/to/workdir", pty=True
-        )
-
-        with DockerRunner(init_docker_client=False, config=config) as dr:
-            args = dr._get_container_kwargs(step, "alpine:3.9", "container_a")
-
-            self.assertEqual(
-                args,
-                {
-                    "image": "alpine:3.9",
-                    "command": ["ls"],
-                    "name": "container_a",
-                    "volumes": [
-                        "/path/to/workdir:/workspace",
-                        "/var/run/docker.sock:/var/run/docker.sock",
-                        "/path/in/host:/path/in/container",
-                    ],
-                    "working_dir": "/tmp/",
-                    "environment": {"FOO": "bar"},
-                    "entrypoint": None,
-                    "detach": False,
-                    "stdin_open": True,
-                    "tty": True,
-                    "privileged": True,
-                    "hostname": "popper.local",
-                    "domainname": "www.example.org",
-                },
-            )
-
-    @unittest.skipIf(os.environ.get("ENGINE", "docker") != "docker", "ENGINE != docker")
-    def test_get_build_info(self):
-        step = Box(
-            {"uses": "popperized/bin/sh@master", "args": ["ls"], "id": "one",},
-            default_box=True,
-        )
-        with DockerRunner(init_docker_client=False) as dr:
-            build, img, tag, build_sources = dr._get_build_info(step)
-            self.assertEqual(build, True)
-            self.assertEqual(img, "popperized/bin")
-            self.assertEqual(tag, "master")
-            self.assertTrue(f"{os.environ['HOME']}/.cache/popper" in build_sources)
-            self.assertTrue("github.com/popperized/bin/sh" in build_sources)
-
-            step = Box(
-                {
-                    "uses": "docker://alpine:3.9",
-                    "runs": ["sh", "-c", "echo $FOO > hello.txt ; pwd"],
-                    "env": {"FOO": "bar"},
-                    "id": "1",
-                },
-                default_box=True,
-            )
-
-        with DockerRunner(init_docker_client=False) as dr:
-            build, img, tag, build_sources = dr._get_build_info(step)
-            self.assertEqual(build, False)
-            self.assertEqual(img, "alpine")
-            self.assertEqual(tag, "3.9")
-            self.assertEqual(build_sources, None)
-
-    @unittest.skipIf(os.environ.get("ENGINE", "docker") != "docker", "ENGINE != docker")
     def test_docker_basic_run(self):
         repo = self.mk_repo()
         conf = ConfigLoader.load(workspace_dir=repo.working_dir)
+        test_string = "STEP_INFO:popper:Successfully tagged popperized/bin:master"
+
+        with WorkflowRunner(conf) as r:
+            wf_data = {"steps": [{"uses": "popperized/bin/sh@master", "args": ["ls"],}]}
+            with self.assertLogs(log, level="STEP_INFO") as cm:
+                r.run(WorkflowParser.parse(wf_data=wf_data))
+            self.assertTrue(test_string in cm.output, f"Got cmd output: {cm.output}")
+
+            wf_data = {
+                "steps": [
+                    {
+                        "uses": "docker://alpine:3.9",
+                        "args": ["sh", "-c", "echo $FOO > hello.txt ; pwd"],
+                        "env": {"FOO": "bar"},
+                    }
+                ]
+            }
+            r.run(WorkflowParser.parse(wf_data=wf_data))
+            with open(os.path.join(repo.working_dir, "hello.txt"), "r") as f:
+                self.assertEqual(f.read(), "bar\n")
+
+            wf_data = {
+                "steps": [
+                    {
+                        "uses": "docker://alpine:3.9",
+                        "args": ["nocommandisnamedlikethis"],
+                    }
+                ]
+            }
+            self.assertRaises(SystemExit, r.run, WorkflowParser.parse(wf_data=wf_data))
+
+        conf = ConfigLoader.load(workspace_dir=repo.working_dir, quiet=True)
+        with WorkflowRunner(conf) as r:
+            wf_data = {"steps": [{"uses": "popperized/bin/sh@master", "args": ["ls"],}]}
+            with self.assertLogs(log, level="STEP_INFO") as cm:
+                r.run(WorkflowParser.parse(wf_data=wf_data))
+            self.assertTrue(test_string not in cm.output)
+
+        repo.close()
+        shutil.rmtree(repo.working_dir, ignore_errors=True)
+
+
+@unittest.skipIf(os.environ.get("ENGINE", "docker") != "podman", "ENGINE != podman")
+class TestHostPodmanRunner(PopperTest):
+    def setUp(self):
+        log.setLevel("CRITICAL")
+
+    def test_stop_running_tasks(self):
+        with PodmanRunner() as pr:
+            cmd = ["podman", "run", "-d", "-q"]
+            _, _, c1 = HostRunner._exec_cmd(
+                cmd + ["debian:buster-slim", "sleep", "20000"], logging=False,
+            )
+            _, _, c2 = HostRunner._exec_cmd(
+                cmd + ["alpine:3.9", "sleep", "10000"], logging=False
+            )
+            c1 = c1.strip()
+            c2 = c2.strip()
+            pr._spawned_containers.add(c1)
+            pr._spawned_containers.add(c2)
+            pr.stop_running_tasks()
+            status_cmd = [
+                "podman",
+                "container",
+                "inspect",
+                "-f",
+                str("{{.State.Status}}"),
+            ]
+            c1_status_cmd = status_cmd + [c1]
+            c2_status_cmd = status_cmd + [c2]
+            _, _, c1_status = HostRunner._exec_cmd(c1_status_cmd, logging=False)
+            _, _, c2_status = HostRunner._exec_cmd(c2_status_cmd, logging=False)
+            self.assertEqual(c1_status, "exited")
+            self.assertEqual(c2_status, "exited")
+
+    def test_find_container(self):
+        config = ConfigLoader.load()
+        step = Box(
+            {
+                "uses": "docker://alpine:3.9",
+                "runs": ["echo hello"],
+                "id": "kontainer_one",
+            },
+            default_box=True,
+        )
+        cid = pu.sanitized_name(step.id, config.wid)
+        with PodmanRunner(init_podman_client=True, config=config) as pr:
+            c = pr._find_container(cid)
+            self.assertEqual(c, None)
+        with PodmanRunner(init_podman_client=True, config=config) as pr:
+            container = pr._create_container(cid, step)
+            c = pr._find_container(cid)
+            self.assertEqual(c, container)
+            cmd = ["podman", "container", "rm", "-f", cid]
+            HostRunner._exec_cmd(cmd, logging=False)
+
+    def test_create_container(self):
+        config = ConfigLoader.load()
+        step = Box(
+            {
+                "uses": "docker://alpine:3.9",
+                "runs": ["echo hello"],
+                "id": "kontainer_one",
+            },
+            default_box=True,
+        )
+        cid = pu.sanitized_name(step.id, config.wid)
+        with PodmanRunner(init_podman_client=True, config=config) as pr:
+            c = pr._create_container(cid, step)
+            c_status_cmd = [
+                "podman",
+                "container",
+                "inspect",
+                "-f",
+                str("{{.State.Status}}"),
+                c,
+            ]
+            __, _, c_status = HostRunner._exec_cmd(c_status_cmd, logging=False)
+            self.assertEqual(c_status, "configured")
+            cmd = ["podman", "container", "rm", c]
+            HostRunner._exec_cmd(cmd, logging=False)
+        step = Box(
+            {
+                "uses": "docker://alpine:3.9",
+                "runs": ["echo", "hello_world"],
+                "id": "KoNtAiNeR tWo",
+            },
+            default_box=True,
+        )
+        cid = pu.sanitized_name(step.id, config.wid)
+        with PodmanRunner(init_podman_client=True, config=config) as pr:
+            c = pr._create_container(cid, step)
+            c_status_cmd = [
+                "podman",
+                "container",
+                "inspect",
+                "-f",
+                str("{{.State.Status}}"),
+                c,
+            ]
+            __, _, c_status = HostRunner._exec_cmd(c_status_cmd, logging=False)
+            self.assertEqual(c_status, "configured")
+            cmd = ["podman", "container", "rm", c]
+            HostRunner._exec_cmd(cmd, logging=False)
+
+    def test_podman_basic_run(self):
+        repo = self.mk_repo()
+        conf = ConfigLoader.load(engine_name="podman", workspace_dir=repo.working_dir)
 
         with WorkflowRunner(conf) as r:
             wf_data = {"steps": [{"uses": "popperized/bin/sh@master", "args": ["ls"],}]}
@@ -300,17 +345,27 @@ class TestHostDockerRunner(PopperTest):
             }
             self.assertRaises(SystemExit, r.run, WorkflowParser.parse(wf_data=wf_data))
 
+            # wf_data = {
+            #     "steps": [
+            #         {
+            #             "uses": "docker://golang:1.14.4-alpine3.11",
+            #             "args": ["go", "build", "-v", "."],
+            #         }
+            #     ]
+            # }
+            # self.assertRaises(SystemExit, r.run, WorkflowParser.parse(wf_data=wf_data))
+
         repo.close()
         shutil.rmtree(repo.working_dir, ignore_errors=True)
 
 
+@unittest.skipIf(
+    os.environ.get("ENGINE", "docker") != "singularity", "ENGINE != singularity"
+)
 class TestHostSingularityRunner(PopperTest):
     def setUp(self):
         log.setLevel("CRITICAL")
 
-    @unittest.skipIf(
-        os.environ.get("ENGINE", "docker") != "singularity", "ENGINE != singularity"
-    )
     def test_get_recipe_file(self):
         repo = self.mk_repo()
         build_ctx_path = repo.working_dir
@@ -352,9 +407,6 @@ exec /bin/bash "$@"''',
             SystemExit, SingularityRunner._get_recipe_file, build_ctx_path, "sample.sif"
         )
 
-    @unittest.skipIf(
-        os.environ.get("ENGINE", "docker") != "singularity", "ENGINE != singularity"
-    )
     def test_create_container(self):
         config = ConfigLoader.load()
         step_one = Box(
@@ -394,9 +446,6 @@ exec /bin/bash "$@"''',
             )
             os.remove(os.path.join(sr._singularity_cache, cid_two))
 
-    @unittest.skipIf(
-        os.environ.get("ENGINE", "docker") != "singularity", "ENGINE != singularity"
-    )
     def test_setup_singularity_cache(self):
         config = ConfigLoader.load()
         with SingularityRunner(config=config) as sr:
@@ -406,9 +455,6 @@ exec /bin/bash "$@"''',
                 sr._singularity_cache,
             )
 
-    @unittest.skipIf(
-        os.environ.get("ENGINE", "docker") != "singularity", "ENGINE != singularity"
-    )
     def test_get_container_options(self):
         config_dict = {
             "engine": {
@@ -441,40 +487,6 @@ exec /bin/bash "$@"''',
                 ],
             )
 
-    @unittest.skipIf(
-        os.environ.get("ENGINE", "docker") != "singularity", "ENGINE != singularity"
-    )
-    def test_get_build_info(self):
-        step = Box(
-            {"uses": "popperized/bin/sh@master", "args": ["ls"], "name": "one",},
-            default_box=True,
-        )
-        with SingularityRunner() as sr:
-            build, img, build_sources = sr._get_build_info(step)
-            self.assertEqual(build, True)
-            self.assertEqual(img, "popperized/bin")
-            self.assertTrue(f"{os.environ['HOME']}/.cache/popper" in build_sources)
-            self.assertTrue(f"github.com/popperized/bin/sh" in build_sources)
-
-            step = Box(
-                {
-                    "uses": "docker://alpine:3.9",
-                    "runs": ["sh", "-c", "echo $FOO > hello.txt ; pwd"],
-                    "env": {"FOO": "bar"},
-                    "name": "1",
-                },
-                default_box=True,
-            )
-
-        with SingularityRunner() as sr:
-            build, img, build_sources = sr._get_build_info(step)
-            self.assertEqual(build, False)
-            self.assertEqual(img, "docker://alpine:3.9")
-            self.assertEqual(build_sources, None)
-
-    @unittest.skipIf(
-        os.environ.get("ENGINE", "docker") != "singularity", "ENGINE != singularity"
-    )
     def test_singularity_start(self):
         repo = self.mk_repo()
         conf = ConfigLoader.load(
